@@ -4,8 +4,9 @@ import click
 from mpi4py import MPI
 from tornado.ioloop import IOLoop
 from tornado import gen
+from warnings import warn
 
-from distributed import Scheduler, Worker
+from distributed import Scheduler, Nanny, Worker
 from distributed.bokeh.worker import BokehWorker
 from distributed.cli.utils import check_python_3, uri_from_host_port
 from distributed.utils import get_ip_interface
@@ -33,12 +34,16 @@ loop = IOLoop()
 @click.option('--scheduler/--no-scheduler', default=True,
               help=("Whether or not to include a scheduler. "
                     "Use --no-scheduler to increase an existing dask cluster"))
+@click.option('--nanny/--no-nanny', default=True,
+              help="Start workers in nanny process for management")
 @click.option('--bokeh-port', type=int, default=8787,
               help="Bokeh port for visual diagnostics")
+@click.option('--bokeh-worker-port', type=int, default=8789,
+              help="Worker's Bokeh port for visual diagnostics")
 @click.option('--bokeh-prefix', type=str, default=None,
               help="Prefix for the bokeh app")
 def main(scheduler_file, interface, nthreads, local_directory, memory_limit,
-         scheduler, bokeh_port, bokeh_prefix):
+         scheduler, bokeh_port, bokeh_prefix, nanny, bokeh_worker_port):
     if interface:
         host = get_ip_interface(interface)
     else:
@@ -63,13 +68,14 @@ def main(scheduler_file, interface, nthreads, local_directory, memory_limit,
         finally:
             scheduler.stop()
     else:
-        worker = Worker(scheduler_file=scheduler_file,
-                        loop=loop,
-                        name=rank if scheduler else None,
-                        ncores=nthreads,
-                        local_dir=local_directory,
-                        services={'bokeh': BokehWorker},
-                        memory_limit=memory_limit)
+        W = Nanny if nanny else Worker
+        worker = W(scheduler_file=scheduler_file,
+                   loop=loop,
+                   name=rank if scheduler else None,
+                   ncores=nthreads,
+                   local_dir=local_directory,
+                   services={('bokeh', bokeh_worker_port): BokehWorker},
+                   memory_limit=memory_limit)
         addr = uri_from_host_port(host, None, 0)
 
         @gen.coroutine
@@ -93,6 +99,10 @@ def main(scheduler_file, interface, nthreads, local_directory, memory_limit,
 
 def go():
     check_python_3()
+    warn("The dask-mpi command line utility in the `distributed` "
+         "package is deprecated.  "
+         "Please install the `dask-mpi` package instead. "
+         "More information is available at https://mpi.dask.org")
     main()
 
 
