@@ -27,7 +27,7 @@ from .registry import Backend, backends
 from .addressing import parse_host_port, unparse_host_port
 from .core import Comm, Connector, Listener, CommClosedError, FatalCommClosedError
 from .utils import (to_frames, from_frames,
-                    get_tcp_server_address, ensure_concrete_host)
+                    get_tcp_server_address, ensure_concrete_host,)
 
 
 logger = logging.getLogger(__name__)
@@ -433,7 +433,7 @@ class BaseTCPListener(Listener, RequireEncryptionMixin):
                      address, self.contact_address)
         local_address = self.prefix + get_stream_address(stream)
         comm = self.comm_class(stream, local_address, address, self.deserialize)
-        self.comm_handler(comm)
+        yield self.comm_handler(comm)
 
     def get_host_port(self):
         """
