@@ -5,6 +5,8 @@ from collections import deque
 from contextlib import contextmanager
 from datetime import timedelta
 import functools
+from hashlib import md5
+import inspect
 import json
 import logging
 import multiprocessing
@@ -34,10 +36,14 @@ except ImportError:
 
 import dask
 from dask import istask
-from toolz import memoize, valmap
+import toolz
 import tornado
 from tornado import gen
-from tornado.ioloop import IOLoop, PollIOLoop
+from tornado.ioloop import IOLoop
+try:
+    from tornado.ioloop import PollIOLoop
+except ImportError:
+    PollIOLoop = None  # dropped in tornado 6.0
 
 from .compatibility import Queue, PY3, PY2, get_thread_identity, unicode
 from .metrics import time
@@ -112,7 +118,7 @@ def get_fileno_limit():
         return 512
 
 
-@memoize
+@toolz.memoize
 def _get_ip(host, port, family, default):
     # By using a UDP socket, we don't actually try to connect but
     # simply select the local address through which *host* is reachable.
@@ -232,19 +238,17 @@ def sync(loop, func, *args, **kwargs):
     Run coroutine in loop running in separate thread.
     """
     # Tornado's PollIOLoop doesn't raise when using closed, do it ourselves
-    if ((isinstance(loop, PollIOLoop) and getattr(loop, '_closing', False)) or
-        (hasattr(loop, 'asyncio_loop') and loop.asyncio_loop._closed)):
+    if PollIOLoop and ((isinstance(loop, PollIOLoop) and getattr(loop, '_closing', False)) or
+            (hasattr(loop, 'asyncio_loop') and loop.asyncio_loop._closed)):
         raise RuntimeError("IOLoop is closed")
+    try:
+        if loop.asyncio_loop.is_closed():  # tornado 6
+            raise RuntimeError("IOLoop is closed")
+    except AttributeError:
+        pass
 
     timeout = kwargs.pop('callback_timeout', None)
 
-    def make_coro():
-        coro = gen.maybe_future(func(*args, **kwargs))
-        if timeout is None:
-            return coro
-        else:
-            return gen.with_timeout(timedelta(seconds=timeout), coro)
-
     e = threading.Event()
     main_tid = get_thread_identity()
     result = [None]
@@ -257,7 +261,10 @@ def sync(loop, func, *args, **kwargs):
                 raise RuntimeError("sync() called from thread of running loop")
             yield gen.moment
             thread_state.asynchronous = True
-            result[0] = yield make_coro()
+            future = func(*args, **kwargs)
+            if timeout is not None:
+                future = gen.with_timeout(timedelta(seconds=timeout), future)
+            result[0] = yield future
         except Exception as exc:
             error[0] = sys.exc_info()
         finally:
@@ -356,12 +363,11 @@ class LoopRunner(object):
             finally:
                 done_evt.set()
 
-        thread = threading.Thread(target=run_loop,
-                                  name="IO loop")
+        thread = threading.Thread(target=run_loop, name="IO loop")
         thread.daemon = True
         thread.start()
 
-        loop_evt.wait(timeout=1000)
+        loop_evt.wait(timeout=10)
         self._started = True
 
         actual_thread = in_thread[0]
@@ -369,6 +375,8 @@ class LoopRunner(object):
             # Loop already running in other thread (user-launched)
             done_evt.wait(5)
             if not isinstance(start_exc[0], RuntimeError):
+                if not isinstance(start_exc[0], Exception):  # track down infrequent error
+                    raise TypeError("not an exception", start_exc[0])
                 raise start_exc[0]
             self._all_loops[self._loop] = count + 1, None
         else:
@@ -405,7 +413,8 @@ class LoopRunner(object):
             try:
                 self._loop.add_callback(self._loop.stop)
                 self._loop_thread.join(timeout=timeout)
-                self._loop.close()
+                with ignoring(KeyError):  # IOLoop can be missing
+                    self._loop.close()
             finally:
                 self._loop_thread = None
 
@@ -547,6 +556,7 @@ def key_split(s):
 try:
     from functools import lru_cache
 except ImportError:
+    lru_cache = False
     pass
 else:
     key_split = lru_cache(100000)(key_split)
@@ -673,7 +683,7 @@ def silence_logging(level, root='distributed'):
     return old
 
 
-@memoize
+@toolz.memoize
 def ensure_ip(hostname):
     """ Ensure that address is an IP address
 
@@ -798,22 +808,23 @@ def _maybe_complex(task):
             type(task) is dict and any(map(_maybe_complex, task.values())))
 
 
-def str_graph(dsk, extra_values=()):
-    def convert(task):
-        if type(task) is list:
-            return [convert(v) for v in task]
-        if type(task) is dict:
-            return valmap(convert, task)
-        if istask(task):
-            return (task[0],) + tuple(map(convert, task[1:]))
-        try:
-            if task in dsk or task in extra_values:
-                return tokey(task)
-        except TypeError:
-            pass
-        return task
+def convert(task, dsk, extra_values):
+    if type(task) is list:
+        return [convert(v, dsk, extra_values) for v in task]
+    if type(task) is dict:
+        return {k: convert(v, dsk, extra_values) for k, v in task.items()}
+    if istask(task):
+        return (task[0],) + tuple(convert(x, dsk, extra_values) for x in task[1:])
+    try:
+        if task in dsk or task in extra_values:
+            return tokey(task)
+    except TypeError:
+        pass
+    return task
 
-    return {tokey(k): convert(v) for k, v in dsk.items()}
+
+def str_graph(dsk, extra_values=()):
+    return {tokey(k): convert(v, dsk, extra_values) for k, v in dsk.items()}
 
 
 def seek_delimiter(file, delimiter, blocksize):
@@ -1015,7 +1026,7 @@ def import_file(path):
     names_to_import = []
     tmp_python_path = None
 
-    if ext in ('.py'):  # , '.pyc'):
+    if ext in ('.py',):  # , '.pyc'):
         if directory not in sys.path:
             tmp_python_path = directory
         names_to_import.append(name)
@@ -1084,7 +1095,15 @@ def format_bytes(n):
     '12.35 MB'
     >>> format_bytes(1234567890)
     '1.23 GB'
+    >>> format_bytes(1234567890000)
+    '1.23 TB'
+    >>> format_bytes(1234567890000000)
+    '1.23 PB'
     """
+    if n > 1e15:
+        return '%0.2f PB' % (n / 1e15)
+    if n > 1e12:
+        return '%0.2f TB' % (n / 1e12)
     if n > 1e9:
         return '%0.2f GB' % (n / 1e9)
     if n > 1e6:
@@ -1260,7 +1279,10 @@ else:
         if isinstance(frame, _bytes_like):
             return len(frame)
         else:
-            return frame.nbytes
+            try:
+                return frame.nbytes
+            except AttributeError:
+                return len(frame)
 
 
 def PeriodicCallback(callback, callback_time, io_loop=None):
@@ -1345,28 +1367,6 @@ class DequeHandler(logging.Handler):
             inst.clear()
 
 
-def fix_asyncio_event_loop_policy(asyncio):
-    """
-    Work around https://github.com/tornadoweb/tornado/issues/2183
-    """
-    class PatchedDefaultEventLoopPolicy(asyncio.DefaultEventLoopPolicy):
-
-        def get_event_loop(self):
-            """Get the event loop.
-
-            This may be None or an instance of EventLoop.
-            """
-            try:
-                return super().get_event_loop()
-            except RuntimeError:
-                # "There is no current event loop in thread"
-                loop = self.new_event_loop()
-                self.set_event_loop(loop)
-                return loop
-
-    asyncio.set_event_loop_policy(PatchedDefaultEventLoopPolicy())
-
-
 def reset_logger_locks():
     """ Python 2's logger's locks don't survive a fork event
 
@@ -1378,5 +1378,55 @@ def reset_logger_locks():
 
 
 # Only bother if asyncio has been loaded by Tornado
-if 'asyncio' in sys.modules:
-    fix_asyncio_event_loop_policy(sys.modules['asyncio'])
+if 'asyncio' in sys.modules and tornado.version_info[0] >= 5:
+
+    jupyter_event_loop_initialized = False
+
+    if 'notebook' in sys.modules:
+        import traitlets
+        from notebook.notebookapp import NotebookApp
+        jupyter_event_loop_initialized = (
+            traitlets.config.Application.initialized() and
+            isinstance(traitlets.config.Application.instance(), NotebookApp)
+        )
+
+    if not jupyter_event_loop_initialized:
+        import asyncio
+        import tornado.platform.asyncio
+        asyncio.set_event_loop_policy(tornado.platform.asyncio.AnyThreadEventLoopPolicy())
+
+
+def has_keyword(func, keyword):
+    if PY3:
+        return keyword in inspect.signature(func).parameters
+    else:
+        # https://stackoverflow.com/questions/50100498/determine-keywords-of-a-tornado-coroutine
+        if gen.is_coroutine_function(func):
+            func = func.__wrapped__
+        return keyword in inspect.getargspec(func).args
+
+
+if lru_cache:
+    has_keyword = lru_cache(1000)(has_keyword)
+
+
+# from bokeh.palettes import viridis
+# palette = viridis(18)
+palette = ['#440154', '#471669', '#472A79', '#433C84', '#3C4D8A', '#355D8C',
+           '#2E6C8E', '#287A8E', '#23898D', '#1E978A', '#20A585', '#2EB27C',
+           '#45BF6F', '#64CB5D', '#88D547', '#AFDC2E', '#D7E219', '#FDE724']
+
+
+@toolz.memoize
+def color_of(x, palette=palette):
+    h = md5(str(x).encode())
+    n = int(h.hexdigest()[:8], 16)
+    return palette[n % len(palette)]
+
+
+def iscoroutinefunction(f):
+    if gen.is_coroutine_function(f):
+        return True
+    if sys.version_info >= (3, 5) and inspect.iscoroutinefunction(f):
+        return True
+    return False
